--  file    : flowers.ex
--  author  : jiri babor
--  email   : jbabor@paradise.net.nz
--  project : texture mapping demo
--  date    : 99-07-20

----------------------------------------------------------------------
--  Note: for frame speed control, uncomment lines 86 and 97        --
----------------------------------------------------------------------
without warning
without type_check

include graphics.e
include map.e
include machine.e
include image.e

constant
    A=#A0000,           -- video memory
    W=320,              -- width, srceen
    H=200,              -- height, screen
    SIZE=W*H,           -- screen size, bytes
    vs=allocate(SIZE),  -- virtual screen, address
    D=100,              -- cube size
    cube={{-1,-1, 1},{ 1,-1, 1},{ 1, 1, 1},{-1, 1, 1},      -- front face
         {-1,-1,-1},{ 1,-1,-1},{ 1, 1,-1},{-1, 1,-1}}*D/2,  -- back face
    f=5/6,              -- fudge factor to compensate for non-square pixel
    -- cube topology
    sq={{3,7,6,2},{4,3,7,8},{1,2,3,4},{5,1,4,8},{5,6,2,1},{6,5,8,7}},
    d={160,100},    -- plane offset
    p={{{12,5},{132,5},{132,120},{12,120}},
      {{12,135},{132,135},{132,240},{12,240}},
      {{12,245},{152,245},{152,356},{12,356}}}

object o
sequence a,bmp,pal,q,r
atom e,t,tf
integer j,n

procedure set_border_color(integer color)
    -- mode 19: screen border color is normally 16
    sequence regs
    regs = repeat(0, 10)
    regs[REG_AX] = #1001
    regs[REG_BX] = color * 256
    regs = dos_interrupt(#10, regs)
end procedure

function rotate(
    sequence s,     -- sequence of 3-d points: {{x1,y1,z1},..{xn,yn,zn}}
    sequence a      -- angles of rotaions: {ax,ay,az}, in radians
    )
    -- rotate structure by given angles and return new 3-d coordinates

    sequence ca,sa,v
    atom x,y,z

    sa=sin(a)
    ca=cos(a)
    v={}
    for i=1 to length(s) do
        y=s[i][2]*ca[1]-s[i][3]*sa[1]
        z=s[i][2]*sa[1]+s[i][3]*ca[1]
        x=s[i][1]*ca[2]+z*sa[2]
        z=-s[i][1]*sa[2]+z*ca[2]
        v=append(v,{x*ca[3]-y*sa[3], f*(x*sa[3]+y*ca[3]), z})
    end for
    return v
end function

---- main --------------------------------------------------------------

o=read_bitmap("3f.bmp")
pal=o[1]/4
bmp=o[2]

o=graphics_mode(19)
tick_rate(200)
all_palette(pal)
set_border_color(0)

n=0                     -- frame counter
t=time()                -- starting time
a={0,0,0}               -- starting angles
while get_key()=-1 do
--    tf=time()+0.03      -- 33 fps
    mem_set(vs,0,SIZE)  -- clear virtual screen
    q=rotate(cube,a)
    for i=1 to 3 do
        e=q[sq[i][1]][3]+q[sq[i][3]][3]
        if e<0 then j=i else j=i+3 end if
        r={q[sq[j][1]][1..2]+d,q[sq[j][2]][1..2]+d,q[sq[j][3]][1..2]+d,q[sq[j][4]][1..2]+d}
        vmap_image(vs,320,bmp,p[i],r)
    end for
    mem_copy(A,vs,SIZE)
    a+={0.027,0.029,0.0031}
--    while time()<tf do end while    -- waste time
    n+=1
end while
t=time()                -- elapsed time

o=graphics_mode(-1)
printf(1,"%6.2f frames per second\n", n/t)
