--  file    : map.e
--  author  : jiri babor
--  email   : jbabor@paradise.net.nz
--  project : texture mapping
--  date    : 99-07-21
--  version : 0.40

function space(
    integer l,      -- length of output sequence
    integer y,      -- first element value
    integer y2      -- last element value
    )
    -- generate integer sequence with linear (even) distribution
    -- of values between y and y2 limits

    sequence s
    integer d,e,f,dx,dy,mi,my,yi

    if l=1 then     -- to guard against degenerate cases
        return {y}
    elsif y=y2 then
        return repeat(y,l)
    else
        s=repeat(y,l)
        s[l]=y2
        dx=l-1
        if y<y2 then
            dy=y2-y
            yi=1
        else
            dy=y-y2
            yi=-1
        end if
        if dx=dy then
            for i=2 to dx do
                y+=yi
                s[i]=y
            end for
        elsif dx>dy then
            d=2*dy
            f=2*dx
            e=-dx
            for i=2 to l-1 do
                e+=d
                if e>=0 then
                    y+=yi
                    e-=f
                end if
                s[i]=y
            end for
        else        -- dy>=dx
            my=floor(dy/dx)
            mi=my*yi
            d=2*dx
            f=2*(dy-my*dx)
            e=-dx
            for i=2 to dx do
                y+=mi
                e+=f
                if e>=0 then
                    y+=yi
                    e-=d
                end if
                s[i]=y
            end for
        end if
        return s
    end if
end function -- space

global procedure map_image(
    sequence tex,   -- texture bitmap, 2-d sequence
    sequence p,     -- vertices in texture: {{u1,v1},..{un,vn}}
    sequence s      -- polygon to be shown: {{x1,y1},..{xn,yn}}
    )
    -- texture mapping, pixel mode:
    -- map polygon p in image (texture) tex into polygon s
    -- polygon s must be convex : any *horizontal* line has no
    -- more than *two* intersections with sides of such polygon

    sequence u,uu,v,vv,x,xx
    integer i1,i2,k,len,n,rows,u1,u2,v1,v2,x1,x2,y,y1,y2,yi,ymax,ymin

    s=floor(s)          -- integers only
    n=length(s)         -- number of sides

    -- get y extremes
    ymin=s[1][2]
    ymax=ymin
    for i=2 to n do
       if s[i][2]<ymin then ymin=s[i][2] end if
       if s[i][2]>ymax then ymax=s[i][2] end if
    end for

    -- set up edge arrays
    rows=ymax-ymin+1
    xx=repeat({},rows)  -- hold x-values
    uu=xx               -- texture horizontal index
    vv=xx               -- texture vertical index

    -- init & loop
    x2=s[n][1]
    y2=s[n][2]-ymin+1
    u2=p[n][1]
    v2=p[n][2]
    for i=1 to n do
        x1=x2
        x2=s[i][1]
        y1=y2
        y2=s[i][2]-ymin+1
        u1=u2
        u2=p[i][1]
        v1=v2
        v2=p[i][2]
        if y1=y2 then   -- horizontal side
            xx[y1] &= x1
            uu[y1] &= u1
            vv[y1] &= v1
        else            -- non-horizontal side
            if y1>y2 then
                len=y1-y2+1
                yi=-1
            else
                len=y2-y1+1
                yi=1
            end if
            x=space(len,x1,x2)
            u=space(len,u1,u2)
            v=space(len,v1,v2)
            k=1
            for j=y1 to y2-yi by yi do
                xx[j] &= x[k]
                uu[j] &= u[k]
                vv[j] &= v[k]
                k+=1
            end for
        end if
    end for

    -- display
    y=ymin
    if length(xx[1])=1 then
        pixel(tex[vv[1][1]][uu[1][1]],{xx[1][1],y})
        y+=1
        i1=2
    else
        i1=1
    end if
    if length(xx[rows])=1 then
        i2=rows-1
        pixel(tex[vv[rows][1]][uu[rows][1]],{xx[rows][1],ymax})
    else
        i2=rows
    end if
    for i=i1 to i2 do
        x=xx[i]
        x1=x[1]
        x2=x[2]
        if x1<x2 then
            len=x2-x1+1
            u=space(len,uu[i][1],uu[i][2])
            v=space(len,vv[i][1],vv[i][2])
        else
            len=x1-x2+1
            u=space(len,uu[i][2],uu[i][1])
            v=space(len,vv[i][2],vv[i][1])
            x1=x2
        end if
        x=repeat(0,len)
        for j=1 to len do
            x[j]=tex[v[j]][u[j]]
        end for
            pixel(x,{x1,y})
        y+=1
    end for
end procedure -- map_image

global procedure vmap_image(
    atom a,         -- screen address
    integer w,      -- screen width
    sequence tex,   -- texture bitmap, 2-d sequence
    sequence p,     -- vertices in texture: {{u1,v1},..{un,vn}}
    sequence s      -- polygon to be shown: {{x1,y1},..{xn,yn}}
    )
    -- texture mapping for virtual screens:
    -- map polygon p in image (texture) tex into polygon s
    -- polygon s must be convex : any *horizontal* line has no
    -- more than *two* intersections with sides of such polygon

    sequence u,uu,v,vv,x,xx
    integer i1,i2,k,len,n,rows,u1,u2,v1,v2,x1,x2,y1,y2,yi,ymax,ymin

    s=floor(s)          -- integers only
    n=length(s)         -- number of sides

    -- get y extremes
    ymin=s[1][2]
    ymax=ymin
    for i=2 to n do
       if s[i][2]<ymin then ymin=s[i][2] end if
       if s[i][2]>ymax then ymax=s[i][2] end if
    end for

    -- set up edge arrays
    rows=ymax-ymin+1
    xx=repeat({},rows)  -- hold x-values
    uu=xx               -- texture horizontal index
    vv=xx               -- texture vertical index

    -- init & loop
    x2=s[n][1]
    y2=s[n][2]-ymin+1
    u2=p[n][1]
    v2=p[n][2]
    for i=1 to n do
        x1=x2
        x2=s[i][1]
        y1=y2
        y2=s[i][2]-ymin+1
        u1=u2
        u2=p[i][1]
        v1=v2
        v2=p[i][2]
        if y1=y2 then   -- horizontal side
            xx[y1] &= x1
            uu[y1] &= u1
            vv[y1] &= v1
        else            -- non-horizontal side
            if y1>y2 then
                len=y1-y2+1
                yi=-1
            else
                len=y2-y1+1
                yi=1
            end if
            x=space(len,x1,x2)
            u=space(len,u1,u2)
            v=space(len,v1,v2)
            k=1
            for j=y1 to y2-yi by yi do
                xx[j] &= x[k]
                uu[j] &= u[k]
                vv[j] &= v[k]
                k+=1
            end for
        end if
    end for

    -- display
    a+=w*ymin
    if length(xx[1])=1 then
        poke(a+xx[1][1],tex[vv[1][1]][uu[1][1]])
        a+=w
        i1=2
    else
        i1=1
    end if
    if length(xx[rows])=1 then
        i2=rows-1
    else
        i2=rows
    end if
    for i=i1 to i2 do
        x=xx[i]
        x1=x[1]
        x2=x[2]
        if x1<x2 then
            len=x2-x1+1
            u=space(len,uu[i][1],uu[i][2])
            v=space(len,vv[i][1],vv[i][2])
        else
            len=x1-x2+1
            u=space(len,uu[i][2],uu[i][1])
            v=space(len,vv[i][2],vv[i][1])
            x1=x2
        end if
        x=repeat(0,len)
        for j=1 to len do
            x[j]=tex[v[j]][u[j]]
        end for
        poke(a+x1,x)
        a+=w
    end for
    if i2<rows then
        poke(a+xx[rows][1],tex[vv[rows][1]][uu[rows][1]])
    end if
end procedure -- vmap_image
