--  file    : pyramid.ex
--  author  : jiri babor
--  email   : jbabor@paradise.net.nz
--  project : texture mapping
--  date    : 99-07-19
--  version : 0.30 (map.e)

----------------------------------------------------------------------
--  Note: for frame speed control, uncomment lines 81 and 92        --
----------------------------------------------------------------------
without warning
without type_check

include graphics.e
include map.e
include machine.e
include image.e

constant
    A=#A0000,           -- video memory
    W=320,              -- width, srceen
    H=200,              -- height, screen
    SIZE=W*H,           -- screen size, bytes
    vs=allocate(SIZE),  -- virtual screen, address
    p={{01,45},{81,45},{41,108}},
    pyr={{0,53,75},{0,53,-75},{-75,-53,0},{75,-53,0}},
    q={{1,2,3},{4,3,2},{2,1,4},{3,4,1}}

object o
sequence a,bmp,pal,r,s
atom t,tf,z
integer n

procedure set_border_color(integer color)
    -- mode 19: screen border color is normally 16
    sequence regs
    regs = repeat(0, 10)
    regs[REG_AX] = #1001
    regs[REG_BX] = color * 256
    regs = dos_interrupt(#10, regs)
end procedure

function rotate(
    sequence s,     -- sequence of 3-d points: {{x1,y1,z1},..{xn,yn,zn}}
    sequence a      -- angles of rotaions: {ax,ay,az}, in radians
    )
    -- rotate structure by given angles and return new 3-d coordinates

    sequence ca,sa,v
    atom x,y,z

    sa=sin(a)
    ca=cos(a)
    v={}
    for i=1 to length(s) do
        y=s[i][2]*ca[1]-s[i][3]*sa[1]
        z=s[i][2]*sa[1]+s[i][3]*ca[1]
        x=s[i][1]*ca[2]+z*sa[2]
        z=-s[i][1]*sa[2]+z*ca[2]
        v=append(v,{x*ca[3]-y*sa[3], x*sa[3]+y*ca[3], z})
    end for
    return v
end function

---- main --------------------------------------------------------------

o=read_bitmap("jiri.bmp")
pal=o[1]/4
bmp=o[2]
a={0.3,0.5,0.7}

o=graphics_mode(19)
tick_rate(200)
all_palette(pal)
set_border_color(0)

n=0                     -- frame counter
t=time()                -- starting time
while get_key()=-1 do
    mem_set(vs,0,SIZE)  -- clear virtual screen
--    tf=time()+0.03      -- 33 fps
    r=rotate(pyr,a)
    for i=1 to 4 do
        z=r[q[i][1]][3]+r[q[i][2]][3]+r[q[i][3]][3]
        if z<0 then     -- if visible
            s={r[q[i][1]][1..2]+{160,100},r[q[i][2]][1..2]+{160,100},r[q[i][3]][1..2]+{160,100}}
            vmap_image(vs,320,bmp,p,s)
        end if
    end for
    mem_copy(A,vs,SIZE)
    a+={0.027,0.029,0.0031}
--    while time()<tf do end while    -- waste time
    n+=1
end while
t=time()                -- elapsed time

o=graphics_mode(-1)
printf(1,"%6.2f frames per second\n", n/t)
